=begin
#--------------------------------------------------------------------------
Copyright 2009, AlexM
Permission to use, copy, modify, and distribute this software for 
any purpose and without fee is hereby granted, provided something the 
above copyright notice appear in all copies.
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#--------------------------------------------------------------------------
Name			:	ComponentDefinition-delete.rb
Description 	:	Method to Delete a ComponentDefinition & its Instances...
Usage			:	Put this file into plugins folder.
					Call it in other scripts.
					It adds a method to Sketchup's ComponentDefinition Class.
Date			:	30 03 2009 - version 1.2
#--------------------------------------------------------------------------
=end

class Sketchup::Group
	def definition
		self.entities.parent
	end
end
#--------------------------------------------------------------------------
if Sketchup.version.to_i >= 7
#--------------------------------------------------------------------------

class Sketchup::ComponentDefinition
	def delete(operation=true)
		model = Sketchup.active_model
		path = model.active_path
		if path then path.reverse!.map!{|c| c.definition}
			if path.include?(self) then model.selection.clear
				(path.index(self)+1).times{model.close_active}
			end
		end
		model.start_operation "Delete Definition of #{self.name}", true if operation
		begin 
			self.entities.erase_entities self.entities.to_a
			operation ? model.commit_operation : true
		rescue
			model.abort_operation if operation
			false
		end
	end#def
end#class

#--------------------------------------------------------------------------
elsif Sketchup.version.to_i == 6
#--------------------------------------------------------------------------

class Sketchup::ComponentDefinition
	def delete(operation=true)
		model = Sketchup.active_model
		path = []
		tr = Geom::Transformation.new.to_a
		definit = model.active_entities.parent
		while definit != model
			if definit.group?
				path << definit.instances.at(0)
				definit = definit.instances.at(0).parent
			else
				i=0
				definit.instances.each do |inst|
				if inst.transformation.to_a == tr && i==0
					path << inst
					i+=1
				elsif i==2
					model.selection.clear
					while model.active_entities.at(0).parent != model
						model.close_active
					end
					path.clear
					break
				end
				end#do
				definit = path.at(-1).parent
			end
		end#while
		if path.any? then path.map!{|c| c.definition}
			if path.include?(self) then model.selection.clear
				(path.index(self)+1).times{model.close_active}
			end
		end
		model.start_operation "Delete Definition of #{self.name}" if operation
		begin 
			self.entities.erase_entities self.entities.to_a
			operation ? model.commit_operation : true
		rescue
			model.abort_operation if operation
			false
		end
	end#def
end#class

#--------------------------------------------------------------------------
elsif Sketchup.version.to_i == 5
#--------------------------------------------------------------------------

class Sketchup::ComponentDefinition
	def delete(operation=true)
		model = Sketchup.active_model
		path = []
		tr = Geom::Transformation.new.to_a
		fpath = proc do |d|
			if d != model
				if d.group?
					path << d.instances.at(0)
					fpath.call(d.instances.at(0).parent)
				else
					instan = []
					d.instances.each do |inst|
						if inst.transformation.to_a == tr
							instan << inst
						end
					end#do
					path = path + instan
					instan.each {|i| fpath.call(i.parent)}
				end
			end
		end
		fpath.call(model.active_entities.parent)
		if path.any? then path.map!{|c| c.definition}
			if path.include?(self) then return false; end
		end
		model.start_operation "Delete Definition of #{self.name}" if operation
		begin 
			self.entities.erase_entities self.entities.to_a
			operation ? model.commit_operation : true
		rescue
			model.abort_operation if operation
			false
		end
	end#def
end#class

end#Sketchup-version
#__END__
#--------------------------------------------------------------------------
#Examples:
#--------------------------------------------------------------------------
require 'sketchup.rb'

def erase_component
	model = Sketchup.active_model
	definitions = model.definitions
	unless definitions.any? 
		UI.messagebox "No groups/components in model!"
		return nil
	end
	names = []
	definitions.each{ |c| names << c.name } #unless c.group?
	tmp = ""*names.max.size
	ret = UI.inputbox(["Components:   "], [tmp], [([tmp]+names.sort).join('|')], "Delete component or group")
	return if not ret || ret == tmp
	if definitions[ret.at(0)].delete
		UI.messagebox "Delete OK!", 0x40
	else
		UI.messagebox "Delete Failure!", 0x110
	end
end#def

file_name = File.basename(__FILE__)
if not file_loaded?(file_name)
	UI.menu("Plugins").add_item("Delete component or group") {erase_component}
	file_loaded(file_name)
end
file_name = nil
